C	*************************************************************
C	ROUTINE TO READ OPTICAL CONSTANTS OF ONE MEDIUM FROM DISK
C	*************************************************************
	SUBROUTINE READCNST
	INTEGER CN,I,IR
	INTEGER M(6)
	REAL L(6,256),N(6,256),K(6,256),MUR(6,256),MUI(6,256)
	REAL LH(256),NH(256),KH(256),MURH(256),MUIH(256)
	REAL LMIN(6),LMAX(6)
	CHARACTER*10 FNAME(6)
	COMMON /OPTCST/ L,N,K,MUR,MUI
	COMMON /ANZ/ M
	COMMON /MINMAX/ LMIN,LMAX
	COMMON /FNAMES/ FNAME

18	FORMAT (' Material :',$)
19	FORMAT (' Type 0-6 !')
340	WRITE (6,18)
	READ (5,*) CN
	IF(CN.EQ.0) RETURN
	IF(CN.GT.0.AND.CN.LT.7) GOTO 330
	WRITE (6,19)
	GOTO 340
330	CALL READMUFILE(LH,NH,KH,MURH,MUIH,M(CN),FNAME(CN))
C	*************************************************************
C	CHECK THE CORRECTNESS OF THE DATA SEQUENCE
C	*************************************************************
	IF(LH(1).LE.LH(M(CN))) THEN
	LMIN(CN)=LH(1)
	LMAX(CN)=LH(M(CN))
	DO 300 I=1,M(CN)
	L(CN,I)=LH(I)
	N(CN,I)=NH(I)
	K(CN,I)=KH(I)
	MUR(CN,I)=MURH(I)
300	MUI(CN,I)=MUIH(I)
	ELSE
310	LMIN(CN)=LH(M(CN))
	LMAX(CN)=LH(1)
	DO 320 I=1,M(CN)
	IR=M(CN)+1-I
	L(CN,I)=LH(IR)
	N(CN,I)=NH(IR)
	K(CN,I)=KH(IR)
	MUR(CN,I)=MURH(IR)
320	MUI(CN,I)=MUIH(IR)
	ENDIF
	RETURN
	END

C	*************************************************************
C	ROUTINE TO SAVE A CALCULATED SPECTRUM OF OPTICAL CONSTANTS
C	ON DISK
C	*************************************************************
	SUBROUTINE SAVECNST
	INTEGER CN,I
	INTEGER M(6)
	REAL L(6,256),N(6,256),K(6,256),MUR(6,256),MUI(6,256)
	REAL LH(256),NH(256),KH(256),MURH(256),MUIH(256)
	CHARACTER*10 FNAME(6)
	COMMON /OPTCST/ L,N,K,MUR,MUI
	COMMON /ANZ/ M
	COMMON /FNAMES/ FNAME

18	FORMAT (' Material :',$)
19	FORMAT (' Type 0-6 !')
400	WRITE (6,18)
	READ (5,*) CN
	IF(CN.EQ.0) RETURN
	IF(CN.GT.0.AND.CN.LT.7) GOTO 410
	WRITE (6,19)
	GOTO 400
410	DO 420 I=1,M(CN)
	LH(I)=L(CN,I)
	NH(I)=N(CN,I)
	KH(I)=K(CN,I)
	MURH(I)=MUR(CN,I)
420	MUIH(I)=MUI(CN,I)
	CALL SAVEMUFILE(LH,NH,KH,MURH,MUIH,M(CN),FNAME(CN))
	RETURN
	END

C	***********************************************************
C	ROUTINES TO READ THE FILE CONTAINING THE OPTICAL CONSTANTS
C	OF ONE MEDIUM
C	***********************************************************
	SUBROUTINE READMUFILE(ZI,NI,KI,MURI,MUII,M,NAME)
	INTEGER LOGI,M,ML,I
	REAL LI(256),KI(256),ZI(256),NI(256),MURI(256),MUII(256),HLP
	CHARACTER*64 LFILE,LFILEM
	CHARACTER*10 NAME
	CHARACTER YN

11	FORMAT (' Name of file of optical constants: ',$)
14	FORMAT (' This file has an illegal extension ! ',/)
30	FORMAT (A64)
100	WRITE (6,11)
	READ (5,30) LFILE
	CALL NAMECHEC(LFILE,NAME,LOGI)
	IF(LOGI.NE.0) GOTO 140
	WRITE (6,14)
	GOTO 100
C	***********************************************************
C	LOADING LINE PARAMETERS FROM FILE CALLED LFILE
C	***********************************************************
140	OPEN(12,ERR=120,FILE=LFILE,STATUS='OLD')
	DO 190 I=1,256
190	READ (12,FMT=*,END=110) ZI(I),NI(I),KI(I)
110	CLOSE (12)
	M=I-1
18	FORMAT (' Is there any permeability file ? ',$)
19	FORMAT (A1)
	WRITE (6,18)
	READ (5,19) YN
	IF (YN.EQ.'y'.OR.YN.EQ.'Y') THEN
C	***********************************************************
C	READING THE PERMEABILITY FILE
C	***********************************************************
	CALL NAMEPERM(LFILE,LFILEM,LOGI)
	OPEN(12,ERR=130,FILE=LFILEM,STATUS='OLD')
	DO 195 I=1,256
195	READ (12,FMT=*,END=115) LI(I),MURI(I),MUII(I)
115	CLOSE (12)
	ML=I-1
C	***********************************************************
C	CHECK CONSISTENCY OF BOTH FILES
C	***********************************************************
	IF (M.NE.ML.OR.ZI(1).NE.LI(1).OR.ZI(M).NE.LI(ML)) GOTO 210	
	ELSE
	DO 310 I=1,M
	MURI(I)=1.0
310	MUII(I)=0.0
	ENDIF
12	FORMAT (' Number of lines in the file(s)   : ',I5)
	WRITE (6,12) M
C       **********************************************************
C       REDUCTION WAVELENGTH, N AND K DEPENDING ON LOGI
C       **********************************************************
        IF (LOGI.LT.0) THEN
        DO 160 I=1,M
160     ZI(I)=1.0/ZI(I)
        ENDIF
        IF (ABS(LOGI).EQ.2) THEN
        DO 320 I=1,M
        HLP=SQRT(NI(I)**2+KI(I)**2)
        KI(I)=SQRT(0.5*(HLP-NI(I)))
320     NI(I)=SQRT(0.5*(HLP+NI(I)))
        ENDIF
        RETURN
C       **********************************************************
C       EXCEPTION HANDLING
C       **********************************************************
13	FORMAT (' There is no access to file       : ',A64,/)
17	FORMAT (' Files of opt.constants and permeability are',
     %  ' inconsistent!')
120	WRITE (6,13) LFILE
	GOTO 100
130	WRITE (6,13) LFILEM
	GOTO 100
210	WRITE (6,17)
	GOTO 100
	END

C	**********************************************************
C	ROUTINE TO WRITE N AND K INTO LFILE
C	**********************************************************
	SUBROUTINE SAVEMUFILE(ZI,NI,KI,MUR,MUI,M,NAME)
	REAL ZI(256),NI(256),KI(256),MUR(256),MUI(256),HLP
	INTEGER LOGI,M,I
	CHARACTER*64 LFILE,LFILEM
	CHARACTER*10 NAME

11	FORMAT (' File name to store optical constants: ',$)
14	FORMAT (' This file has an illegal extension ! ')
30	FORMAT (A64)
100	WRITE (6,11)
	READ (5,30) LFILE
	CALL NAMECHEC(LFILE,NAME,LOGI)
        IF(LOGI.EQ.0) THEN
        WRITE (6,14)
        GOTO 100
        ENDIF
C       *************************************************************
C       TRANSFORMATION DEPENDING ON THE NAMECHECK RESULT
C       *************************************************************
        IF (LOGI.LT.0) THEN
        DO 300 I=1,M
300     ZI(I)=1.0/ZI(I)
        ENDIF
        IF (ABS(LOGI).EQ.2) THEN
        DO 320 I=1,M
        HLP=KI(I)
        KI(I)=2*HLP*NI(I)
320     NI(I)=NI(I)**2-HLP**2
        ENDIF
C	*************************************************************
C	STORAGE PROCESS DEPENDENT FROM THE NAMECHECK RESULT
C	*************************************************************
110	OPEN(12,ERR=150,FILE=LFILE)
21	FORMAT (' ',E11.5,' ',E11.5,' ',E11.5)
        IF(LOGI.LT.0) THEN
	WRITE (12,21) (ZI(I),NI(I),KI(I),I=M,1,-1)
	ELSE
	WRITE (12,21) (ZI(I),NI(I),KI(I),I=1,M)
	ENDIF
140	CLOSE (12)
C	*************************************************************
C	STORE THE PERMEABILITY FILE
C	*************************************************************
	CALL NAMEPERM(LFILE,LFILEM,LOGI)
	OPEN(12,ERR=250,FILE=LFILEM)
        IF(LOGI.LT.0) THEN
	WRITE (12,21) (ZI(I),MUR(I),MUI(I),I=M,1,-1)
	ELSE
	WRITE (12,21) (ZI(I),MUR(I),MUI(I),I=1,M)
	ENDIF
240	CLOSE (12)
	RETURN
C       **********************************************************
C       EXCEPTION HANDLING
C       **********************************************************
13	FORMAT (' There is no access to file       : ',A32)
150	WRITE (6,13) LFILE
	GOTO 100
250	WRITE (6,13) LFILEM
	GOTO 100
	END

C	*************************************************************
C	CONSTRUCTION OF THE NAME OF THE PERMEABILITY FILE
C	*************************************************************
	SUBROUTINE NAMEPERM(FILEN,FILEM,LZ)
	INTEGER EXA,NME,LZ
	CHARACTER*64 FILEN,FILEM,HNAME
	CHARACTER*3 EXTL,EXTZ
	DATA EXTL,EXTZ/'lmu','zmu'/

        HNAME=FILEN
        NME=0
100     EXA=INDEX(HNAME,'.')+1
        IF (EXA.GT.1) THEN
        HNAME=HNAME(EXA:)
        NME=NME+EXA-1
        GOTO 100
        ENDIF

	IF (LZ.GT.0) THEN
	FILEM=FILEN(1:NME)//EXTL
	ELSE
	FILEM=FILEN(1:NME)//EXTZ
	ENDIF
	RETURN
	END

